# Generated by Django 5.0.14 on 2025-07-10 09:00

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbportfolio', '0079_alter_trade_drift_factor'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assetposition',
            name='weighting',
            field=models.DecimalField(decimal_places=8, default=Decimal('0'), help_text='The Weight of the Asset on the price date of the Asset.', max_digits=9, verbose_name='Weight'),
        ),
    ]
