# Generated by Django 5.0.8 on 2024-08-13 13:48

import django.db.models.manager
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbportfolio", "0063_accountreconciliation_accountreconciliationline_and_more"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="portfolio",
            managers=[
                ("tracked_objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AddField(
            model_name="portfolio",
            name="is_tracked",
            field=models.BooleanField(
                default=True,
                help_text="True if the internal updating mechanism (e.g., Propagation, Synchronization etc...) needs to apply to this portfolio",
            ),
        ),
        migrations.AlterField(
            model_name="portfolio",
            name="is_invested",
            field=models.BooleanField(default=False, help_text="True if the portfolio AUM counts"),
        ),
        migrations.AlterField(
            model_name="portfolio",
            name="is_manageable",
            field=models.BooleanField(
                default=False,
                help_text="True if the portfolio can be manually modified (e.g. Trade proposal be submitted or total weight recomputed)",
            ),
        ),
    ]
